#	
#     This R script is part of the bundle used in the paper:
#
#     Modeling tissue contamination to improve molecular identification
#     of the primary tumor site of metastases. 	
#
#     Description of this R script:
#     Scripts for contamination model simulation
#
#     Intended for use with R.
#     Copyright (C) 2013 Martin Vincent
# 
#     This program is free software: you can redistribute it and/or modify
#     it under the terms of the GNU General Public License as published by
#     the Free Software Foundation, either version 3 of the License, or
#     (at your option) any later version.
# 
#     This program is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#     GNU General Public License for more details.
# 
#     You should have received a copy of the GNU General Public License
#     along with this program.  If not, see <http://www.gnu.org/licenses/>
#


 
################################################################
###
### Contamination models
### Computes: f(alpha*f^{-1}(x) + (1-alpha)*f^{-1}(y))
###

# Log model
ct.mix <- function(x, y, alpha, eta = 1.8) sapply(1:length(x), function(i) -1/log(eta)*log(alpha*eta^(-x[i])+(1-alpha)*eta^(-y[i])))

# Linear model
linear.mix <- function(x, y, alpha) sapply(1:length(x), function(i) alpha*x[i]+(1-alpha)*y[i])

################################################################
###
### Compute simulated samples
###

model.simple <- function(n, classwise = FALSE, replace = TRUE, data.x, data.z, alpha, mix.fun = ct.mix) {
	
	if(classwise) {
		
		x.samples <- as.vector(sapply(unique(data.x$Class), function(cls) sample(sampleNames(data.x[,data.x$Class == cls]), size = n, replace = replace)))
		
	} else {
		x.samples <- sample(sampleNames(data.x), size = n, replace = replace)
	}
	
	n <- length(x.samples)
	
	z.samples <- sample(sampleNames(data.z), size = n, replace = TRUE)
	
	m <- sapply(1:n, function(i) mix.fun(exprs(data.x)[, x.samples[i]], exprs(data.z)[, z.samples[i]], alpha[i]))
	
	colnames(m) <- paste("model",1:n, sep="_")
	
	pdata <- data.frame(Class = data.x[,x.samples]$Class, X_ID = x.samples, Z_ID = z.samples, alpha = alpha,  stringsAsFactors = FALSE)
	rownames(pdata) <- colnames(m)
	exprs <- new("ExpressionSet", exprs=m, phenoData = new("AnnotatedDataFrame", data=pdata), featureData = new("AnnotatedDataFrame",fData(data.x)))
	
	return(exprs)
}
