#	
#     This R script is part of the bundle used in the paper:
#
#     Modeling tissue contamination to improve molecular identification
#     of the primary tumor site of metastases. 	
#
#     Description of this R script:
#     Produce a plot similar to Figure 5 in the paper.
#
#     Intended for use with R.
#     Copyright (C) 2013 Martin Vincent
# 
#     This program is free software: you can redistribute it and/or modify
#     it under the terms of the GNU General Public License as published by
#     the Free Software Foundation, either version 3 of the License, or
#     (at your option) any later version.
# 
#     This program is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#     GNU General Public License for more details.
# 
#     You should have received a copy of the GNU General Public License
#     along with this program.  If not, see <http://www.gnu.org/licenses/>
#

library(ggplot2)
library(reshape2)
library(plyr)

#TODO update file path
source('load_data.R')

cls.pca <- levels(classes.prim)
cls.plot <- c("Breast", "CRC", "EG", "Pancreas",  "Squamous", "HCC", "CCA")

#########
###
### Compute pca components
###


x.prim.un <- x.prim.un[classes.prim %in% cls.pca,]
x.sim.un <- x.simnl.un[classes.simnl %in% cls.pca,]
x.cb.un <- x.cb.un[classes.cb %in% cls.pca,]

x.prim <- x.prim[classes.prim %in% cls.pca,]
x.sim <- x.simnl[classes.simnl %in% cls.pca,]
x.cb <- x.cb[classes.cb %in% cls.pca,]

classes.prim <- factor(classes.prim[classes.prim %in% cls.pca])
classes.sim <- factor(classes.simnl[classes.simnl %in% cls.pca])
classes.cb <- factor(classes.cb[classes.cb %in% cls.pca])

xi <- matrix(0, ncol(x.prim.un), length(cls.pca))
for(i in seq_along(cls.pca)) {
  xi[, i] <- apply(x.prim.un[classes.prim == cls.pca[i], ], 2, mean)
} 

xi <- t(scale(t(xi), center = TRUE, scale = FALSE))
center <- attr(xi, "scaled:center")
x.prim.un <- t(t(x.prim.un) - center)
x.sim.un <- t(t(x.sim.un) - center)
x.cb.un <- t(t(x.cb.un) - center)

udv <- svd(t(xi))
colnames(udv$v) <- paste("PC", 1:ncol(udv$v), sep="")

#######################
###
### Projection

pca.prim <- x.prim.un %*% udv$v
pca.sim <- x.sim.un %*% udv$v
pca.cb <- x.cb.un %*% udv$v


########## PCA plots

plot.data1 <- data.frame(pca.sim, type = rep("sim", nrow(x.sim.un)), class = as.character(classes.sim))
plot.data2 <- data.frame(rbind(pca.prim, pca.cb), type = c(rep("Resections", nrow(x.prim.un)), rep("Liver core biopsies", nrow(x.cb))), class = c(as.character(classes.prim), as.character(classes.cb)))

theme_set(theme_bw())

fig5 <- ggplot(subset(plot.data1, class %in% cls.plot), aes(x = PC1, y = PC2)) + 
  geom_hex() +
  scale_fill_continuous(low = "gray90", high = "gray50", guide = FALSE) +
  geom_point(data = subset(plot.data2, class %in% cls.plot), aes(shape = type), size = 2) +
  scale_shape_manual(name = "Samples", values=c('Liver core biopsies' = 19, 'Resections' = 4)) + 
  xlab("First principal component") + 
  ylab("Second principal component") +
  theme(legend.position = "bottom", legend.direction='horizontal', legend.key = element_blank(), strip.text.x = element_text(size = 12), strip.background = element_blank())


print(fig5)
