#	
#     This R script is part of the bundle used in the paper:
#
#     Modeling tissue contamination to improve molecular identification
#     of the primary tumor site of metastases. 	
#
#     Description of this R script:
#     Scrip for loading data, running simulations, normalizing and standardizing data.
#
#     Intended for use with R.
#     Copyright (C) 2013 Martin Vincent
# 
#     This program is free software: you can redistribute it and/or modify
#     it under the terms of the GNU General Public License as published by
#     the Free Software Foundation, either version 3 of the License, or
#     (at your option) any later version.
# 
#     This program is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#     GNU General Public License for more details.
# 
#     You should have received a copy of the GNU General Public License
#     along with this program.  If not, see <http://www.gnu.org/licenses/>
#


################################################################
###
### Load libraries and scripts
###

library(Biobase)

# Load contamination model and simulation scrips 

#TODO update file path
source("contamination_model_simulation.R")

# Define various functions

loglik_sum <- function(res, classes, w = rep(1, length(classes))) sum(w*log(sapply(1:length(classes), function(i) res[classes[i],i])))

argmax <- function(x) which(max(x) == x)
argmin <- function(x) which(min(x) == x)

sum.list <- function(x) {

  tmp <- x[[1]]

  if(length(x) == 1) {
    return(tmp)
  }

  for(i in 2:length(x)) {
    tmp <- tmp + x[[i]]
  }

  return(tmp)
}

mean.list <- function(x) sum.list(x)/length(x)



################################################################
###
### Settings
###

# SEED
set.seed(19)

# Build settings

build <- list()
build$alpha <- 0 		# Alpha used in msgl (0 - group lasso, 1 - lasso, between 0 and 1 sparse group lasso)
build$lambdamin <- 1e-3 	# minimum lambda used in msgl
build$d <- 150L 		# number of estimated models, used in msgl
build$miR.cutoff <- .85		# Cutoff threshold
build$M <- 750L 		# Number of simulated samples per class (default 750)
build$classwise <- TRUE		 
build$replace <- TRUE		 



################################################################
###
### Load data
###

#TODO update file path
load('expression_set_data.rd')
data <- exprs

# Extract subset
data <- exprs[,exprs$Class %in% c("Breast", "CRC", "EG", "Liver", "Pancreas",  "Squamous", "Cirrhosis", "CCA", "HCC")]

# Primary cancer samples
data.prim <- data[, data$Sample.type %in% c("Res", "ResM")]
x.prim.un <- t(exprs(data.prim))
colnames(x.prim.un) <- fData(data.prim)$Target.shortname
classes.prim <- factor(data.prim$Class)

# Core-biopsy samples 
data.cb <- data[, data$Sample.type %in% c("CB")]
x.cb.un <- t(exprs(data.cb))
colnames(x.cb.un) <- fData(data.cb)$Target.shortname
classes.cb <- factor(data.cb$Class)



################################################################
###
### Simulate data
###

M <- build$M

data.livercirr.sim <- data.prim[, data.prim$Class %in% c("Liver", "Cirrhosis")]
data.liver.sim <- data.prim[, data.prim$Class =="Liver"]
data.cancer <- data.prim[,!(data.prim$Class %in% c("Liver", "Cirrhosis"))]

data.liverback.sim <- data.liver.sim

if(build$classwise) {
	N <- M*length(unique(data.cancer$Class))
} else {
	N <- M
}

# Non linear contamination model
data.simnl <- model.simple(n=M, classwise = build$classwise, replace = build$replace, data.cancer, data.liverback.sim, alpha = rbeta(N, 2, 2))

# Linear contamination model
data.simli <- model.simple(n=M, classwise = build$classwise, replace = build$replace, data.cancer, data.liverback.sim, alpha = rbeta(N, 2, 2), mix.fun = linear.mix)

# Extract design matrix and class vector for simulated samples
pData(data.livercirr.sim) <- data.frame(Class = data.livercirr.sim$Class, X_ID = as.character(data.livercirr.sim$ID_A), Z_ID = as.character(NA), alpha =as.numeric(NA), stringsAsFactors = FALSE)

data.simnl <- combine(data.simnl, data.livercirr.sim)
data.simli <- combine(data.simli, data.livercirr.sim)

x.simnl.un <- t(exprs(data.simnl))
colnames(x.simnl.un) <- fData(data.simnl)$Target.shortname
classes.simnl <- factor(data.simnl$Class)

x.simli.un <- t(exprs(data.simli))
colnames(x.simli.un) <- fData(data.simli)$Target.shortname
classes.simli <- factor(data.simli$Class)



################################################################
###
### Set samples names and print number of samples
###

names(classes.cb) <- rownames(x.cb.un)
names(classes.prim) <- rownames(x.prim.un)
names(classes.simnl) <- rownames(x.simnl.un)
names(classes.simli) <- rownames(x.simli.un)

# Print number of samples
print(table(classes.prim))
print(table(classes.cb))
print(table(classes.simnl))
print(table(classes.simli))



################################################################
###
### Remove controls and non expressed miRNAs
###

tmp <- sapply(levels(classes.prim), function(cls) colSums(x.prim.un[classes.prim == cls, ] == 40))
sel <- which(apply(t(t(tmp)/as.numeric(table(classes.prim))), 1, function(x) all(x > build$miR.cutoff)))
sel <- c(sel, which(colnames(x.prim.un) %in% c("MammU6", "ath", "RNU44", "RNU48", "miR.672", "miR.674", "miR.871", "miR.872", "miR.886.3p", "miR.886.5p")))

print(colnames(x.cb.un)[sel])
#save(sel, file="miRFilter.rd")

x.prim.un <- x.prim.un[,-sel]
x.cb.un <- x.cb.un[,-sel]
x.simnl.un <- x.simnl.un[,-sel]
x.simli.un <- x.simli.un[,-sel]



################################################################
###
### Normalization and standardization
###

# Normalization

x.prim <- t(scale(t(x.prim.un), center = TRUE, scale = TRUE))
x.cb <- t(scale(t(x.cb.un), center = TRUE, scale = TRUE))
x.simnl <- t(scale(t(x.simnl.un), center = TRUE, scale = TRUE))
x.simli <- t(scale(t(x.simli.un), center = TRUE, scale = TRUE))

# Standardization

x.center <-  colMeans(x.prim)
x.scale <- apply(x.prim, 2, sd)

x.prim <- t((t(x.prim) - x.center)/x.scale)
x.cb <- t((t(x.cb) - x.center)/x.scale)
x.simnl <- t((t(x.simnl) - x.center)/x.scale)
x.simli <- t((t(x.simli) - x.center)/x.scale)
