#	
#     This R script is part of the bundle used in the paper:
#
#     Modeling tissue contamination to improve molecular identification
#     of the primary tumor site of metastases. 	
#
#     Description of this R script:
#     Validation of multinomial group lasso predictor, 
#     based on a combination of the resections and core biopsies
#
#     Intended for use with R.
#     Copyright (C) 2013 Martin Vincent
# 
#     This program is free software: you can redistribute it and/or modify
#     it under the terms of the GNU General Public License as published by
#     the Free Software Foundation, either version 3 of the License, or
#     (at your option) any later version.
# 
#     This program is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#     GNU General Public License for more details.
# 
#     You should have received a copy of the GNU General Public License
#     along with this program.  If not, see <http://www.gnu.org/licenses/>
#

library(msgl)
library(subsamples)

#TODO update file path
source('load_data.R')

results <- NULL

cls.list <- list()
cls.list$non.liver <- c("Breast", "CRC", "EG", "Pancreas",  "Squamous")
cls.list$liver <- c("Liver", "Cirrhosis", "CCA", "HCC")

tp.cb <- cut(as.numeric(sub("%", "", data.cb$tumor))/100, breaks = c(0,1/2, 1))

### Sub samples

number.of.subsamples <- 100L 

subs.test <- sub.samples(classes.cb, d=number.of.subsamples, fraction = .50)

standardize <- FALSE


for(n in c(1L, 2L, 3L, 4L)) {

print(n)

if(n == 0) {
  subs.train <- lapply(subs.test, function(s) c())
} else {
  subs.train <- lapply(subs.test, function(s) which(names(classes.cb) %in% names(classes.cb[-s])[classes.cb[-s] %in% cls.list$non.liver][sub.samples(classes.cb[-s][classes.cb[-s] %in% cls.list$non.liver], d=1L, n = n)[[1]]]))
#lapply(subs.test, function(s) which(names(classes.cb) %in% names(classes.cb[-s])[sub.samples(classes.cb[-s], d=1L, n = n)[[1]]]))
}

### Check that there is no bug in the code for sampleing test and traning sets 
lapply(1:length(subs.test), function(i) if(any(subs.test[[i]] %in% subs.train[[i]])) stop("Error - test samples found in training samples"))



############################
###
### PRIM+CB
###

print("PRIM+CB")

# Sample weights
#sW.cb <- rep(1/(length(classes.cb), length(classes.cb))
#sW.prim <- rep(1/length(classes.prim), length(classes.prim)) #1/length(levels(classes.prim))*sapply(classes.prim, function(cls) 1/sum(classes.prim == cls))
sW <- rep(1/(length(classes.cb)+length(classes.prim)), length(classes.cb)+length(classes.prim)) #c(sW.cb, sW.prim)

# Combine data
x.comb <- rbind(x.cb, x.prim)
classes.comb <- factor(c(as.character(classes.cb), as.character(classes.prim)))

test.samples <- subs.test
training.samples <- lapply(subs.train, function(x) c(x, 1:length(classes.prim)+length(classes.cb)))

### Double check that there is no bug in the code for sampleing test and traning sets 
lapply(1:length(test.samples), function(i) if(any(test.samples[[i]] %in% training.samples[[i]])) stop("Error - test samples found in training samples"))

lambda <- msgl.lambda.seq(x.comb, classes.comb, sampleWeights = sW, alpha = build$alpha, d = build$d, standardize = standardize, lambda.min = build$lambdamin)

fit.sub <- msgl.subsampling(x.comb, classes.comb, sampleWeights = sW, lambda = lambda, alpha = build$alpha, training = training.samples, test = test.samples, standardize = standardize, max.threads = 8L)


err.matrix <- sapply(1:length(fit.sub$classes), function(i) colSums(fit.sub$classes[[i]] != classes.cb[test.samples[[i]]])/length(test.samples[[i]]))
loglike.matrix <- sapply(1:length(fit.sub$classes), function(i) sapply(fit.sub$response[[i]], function(res) -loglik_sum(res, classes.cb[test.samples[[i]]])))

err <- rowMeans(err.matrix)
features <- colMeans(fit.sub$features)
loglike <- rowMeans(loglike.matrix)

results <- rbind(results, data.frame(class = "all", err = err, features = features, loglike = loglike, n=n, classifier = "PRIM + CB"))

for(j in 1:length(cls.list)) {

err.matrix <- sapply(1:length(fit.sub$classes), function(i) colSums(fit.sub$classes[[i]][classes.cb[test.samples[[i]]] %in% cls.list[[j]],] != classes.cb[test.samples[[i]]][classes.cb[test.samples[[i]]] %in% cls.list[[j]]])/sum(classes.cb[test.samples[[i]]] %in% cls.list[[j]]))
loglike.matrix <- sapply(1:length(fit.sub$classes), function(i) sapply(fit.sub$response[[i]], function(res) -loglik_sum(res[,classes.cb[test.samples[[i]]] %in% cls.list[[j]]], classes.cb[test.samples[[i]]][classes.cb[test.samples[[i]]] %in% cls.list[[j]]])))

err <- rowMeans(err.matrix)
features <- colMeans(fit.sub$features)
loglike <- rowMeans(loglike.matrix)

results <- rbind(results, data.frame(class = names(cls.list)[j], err = err, features = features, loglike = loglike, n= n, classifier = "PRIM + CB"))
}

## Tp errors
err.tp <- list()
for(i in 1:length(test.samples)) {
  sel.tmp <- classes.cb[test.samples[[i]]] %in% cls.list$non.liver
  errors.matrix <- fit.sub$classes[[i]][sel.tmp, ] != classes.cb[test.samples[[i]]][sel.tmp]
  err.tp[[i]] <- sapply(levels(tp.cb), function(q) colMeans(errors.matrix[tp.cb[test.samples[[i]]][sel.tmp] == q, ]))
}

err.tp <- sum.list(err.tp)/length(err.tp)
results <- rbind(results, data.frame(class = "low", err = err.tp[,1], features = features, loglike = loglike, n= n, classifier = "PRIM + CB"))
results <- rbind(results, data.frame(class = "high", err = err.tp[,2], features = features, loglike = loglike, n= n, classifier = "PRIM + CB"))


############################
###
### SIM non linear + CB Classfier
###

print("SIM NL + CB")

x.comb <- rbind(x.cb, x.simnl)
classes.comb <-  factor(c(as.character(classes.cb), as.character(classes.simnl)))

test.samples <- subs.test
training.samples <- lapply(subs.train, function(x) c(x, 1:length(classes.simnl)+length(classes.cb)))

sW.simnl <- sapply(classes.simnl, function(cls) sum(classes.prim == cls)/(length(classes.prim)+length(classes.cb))/sum(classes.simnl == cls))
sW.cb <-  rep(1/(length(classes.prim)+length(classes.cb)), length(classes.cb))
sW <- c(sW.cb, sW.simnl)

### Double check that there is no bug in the code for sampleing test and traning sets 
lapply(1:length(test.samples), function(i) if(any(test.samples[[i]] %in% training.samples[[i]])) stop("Error - test samples found in training samples"))

lambda <- msgl.lambda.seq(x.comb, classes.comb, sampleWeights = sW, alpha = build$alpha, d = build$d, standardize = standardize, lambda.min = build$lambdamin)
fit.sub <- msgl.subsampling(x.comb, classes.comb, sampleWeights = sW, lambda = lambda, alpha = build$alpha, standardize = standardize, training = training.samples, test = test.samples, max.threads = 8L)

err.matrix <- sapply(1:length(fit.sub$classes), function(i) colSums(fit.sub$classes[[i]] != classes.cb[test.samples[[i]]])/length(test.samples[[i]]))
loglike.matrix <- sapply(1:length(fit.sub$classes), function(i) sapply(fit.sub$response[[i]], function(res) -loglik_sum(res, classes.cb[test.samples[[i]]])))

err <- rowMeans(err.matrix)
features <- colMeans(fit.sub$features)
loglike <- rowMeans(loglike.matrix)

results <- rbind(results, data.frame(class = "all", err = err, features = features, loglike = loglike, n=n, classifier = "SIM NL + CB"))

for(j in 1:length(cls.list)) {

err.matrix <- sapply(1:length(fit.sub$classes), function(i) colSums(fit.sub$classes[[i]][classes.cb[test.samples[[i]]] %in% cls.list[[j]],] != classes.cb[test.samples[[i]]][classes.cb[test.samples[[i]]] %in% cls.list[[j]]])/sum(classes.cb[test.samples[[i]]] %in% cls.list[[j]]))
loglike.matrix <- sapply(1:length(fit.sub$classes), function(i) sapply(fit.sub$response[[i]], function(res) -loglik_sum(res[,classes.cb[test.samples[[i]]] %in% cls.list[[j]]], classes.cb[test.samples[[i]]][classes.cb[test.samples[[i]]] %in% cls.list[[j]]])))

err <- rowMeans(err.matrix)
features <- colMeans(fit.sub$features)
loglike <- rowMeans(loglike.matrix)

results <- rbind(results, data.frame(class = names(cls.list)[j], err = err, features = features, loglike = loglike, n= n, classifier = "SIM NL + CB"))
}


## Tp errors
err.tp <- list()
for(i in 1:length(test.samples)) {
  sel.tmp <- classes.cb[test.samples[[i]]] %in% cls.list$non.liver
  errors.matrix <- fit.sub$classes[[i]][sel.tmp, ] != classes.cb[test.samples[[i]]][sel.tmp]
  err.tp[[i]] <- sapply(levels(tp.cb), function(q) colMeans(errors.matrix[tp.cb[test.samples[[i]]][sel.tmp] == q, ]))
}

err.tp <- sum.list(err.tp)/length(err.tp)
results <- rbind(results, data.frame(class = "low", err = err.tp[,1], features = features, loglike = loglike, n= n, classifier = "SIM NL + CB"))
results <- rbind(results, data.frame(class = "high", err = err.tp[,2], features = features, loglike = loglike, n= n, classifier = "SIM NL + CB"))



############################
###
### SIM linear + CB Classfier
###

print("SIM LI + CB")

x.comb <- rbind(x.cb, x.simli)
classes.comb <-  factor(c(as.character(classes.cb), as.character(classes.simli)))

test.samples <- subs.test
training.samples <- lapply(subs.train, function(x) c(x, 1:length(classes.simli)+length(classes.cb)))

sW.simli <- sapply(classes.simli, function(cls) sum(classes.prim == cls)/(length(classes.prim)+length(classes.cb))/sum(classes.simli == cls))
sW.cb <- rep(1/(length(classes.prim)+length(classes.cb)), length(classes.cb))
sW <- c(sW.cb, sW.simli)

### Double check that there is no bug in the code for sampleing test and traning sets 
lapply(1:length(test.samples), function(i) if(any(test.samples[[i]] %in% training.samples[[i]])) stop("Error - test samples found in training samples"))

lambda <- msgl.lambda.seq(x.comb, classes.comb, sampleWeights = sW, alpha = build$alpha, d = build$d, standardize = standardize, lambda.min = 1e-3)
fit.sub <- msgl.subsampling(x.comb, classes.comb, sampleWeights = sW, lambda = lambda, alpha = build$alpha, standardize = FALSE, training = training.samples, test = test.samples, max.threads = 8L)

err.matrix <- sapply(1:length(fit.sub$classes), function(i) colSums(fit.sub$classes[[i]] != classes.cb[test.samples[[i]]])/length(test.samples[[i]]))
loglike.matrix <- sapply(1:length(fit.sub$classes), function(i) sapply(fit.sub$response[[i]], function(res) -loglik_sum(res, classes.cb[test.samples[[i]]])))

err <- rowMeans(err.matrix)
features <- colMeans(fit.sub$features)
loglike <- rowMeans(loglike.matrix)

results <- rbind(results, data.frame(class = "all", err = err, features = features, loglike = loglike, n=n, classifier = "SIM LI + CB"))

for(j in 1:length(cls.list)) {

err.matrix <- sapply(1:length(fit.sub$classes), function(i) colSums(fit.sub$classes[[i]][classes.cb[test.samples[[i]]] %in% cls.list[[j]],] != classes.cb[test.samples[[i]]][classes.cb[test.samples[[i]]] %in% cls.list[[j]]])/sum(classes.cb[test.samples[[i]]] %in% cls.list[[j]]))
loglike.matrix <- sapply(1:length(fit.sub$classes), function(i) sapply(fit.sub$response[[i]], function(res) -loglik_sum(res[,classes.cb[test.samples[[i]]] %in% cls.list[[j]]], classes.cb[test.samples[[i]]][classes.cb[test.samples[[i]]] %in% cls.list[[j]]])))

err <- rowMeans(err.matrix)
features <- colMeans(fit.sub$features)
loglike <- rowMeans(loglike.matrix)

results <- rbind(results, data.frame(class = names(cls.list)[j], err = err, features = features, loglike = loglike, n= n, classifier = "SIM LI + CB"))
}


## Tp errors
err.tp <- list()
for(i in 1:length(test.samples)) {
  sel.tmp <- classes.cb[test.samples[[i]]] %in% cls.list$non.liver
  errors.matrix <- fit.sub$classes[[i]][sel.tmp, ] != classes.cb[test.samples[[i]]][sel.tmp]
  err.tp[[i]] <- sapply(levels(tp.cb), function(q) colMeans(errors.matrix[tp.cb[test.samples[[i]]][sel.tmp] == q, ]))
}

err.tp <- sum.list(err.tp)/length(err.tp)
results <- rbind(results, data.frame(class = "low", err = err.tp[,1], features = features, loglike = loglike, n= n, classifier = "SIM LI + CB"))
results <- rbind(results, data.frame(class = "high", err = err.tp[,2], features = features, loglike = loglike, n= n, classifier = "SIM LI + CB"))

}

