#	
#     This R script is part of the bundle used in the paper:
#
#     Modeling tissue contamination to improve molecular identification
#     of the primary tumor site of metastases. 	
#
#     Description of this R script:
#     Validation of multinomial group lasso predictors based solely on resections
#
#     Intended for use with R.
#     Copyright (C) 2013 Martin Vincent
# 
#     This program is free software: you can redistribute it and/or modify
#     it under the terms of the GNU General Public License as published by
#     the Free Software Foundation, either version 3 of the License, or
#     (at your option) any later version.
# 
#     This program is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#     GNU General Public License for more details.
# 
#     You should have received a copy of the GNU General Public License
#     along with this program.  If not, see <http://www.gnu.org/licenses/>
#

library(msgl)

#TODO update file path
source('load_data.R')

cls.list <- list()
cls.list$non.liver <- c("Breast", "CRC", "EG", "Pancreas",  "Squamous")
cls.list$liver <- c("Liver", "Cirrhosis", "CCA", "HCC")

tp.cb <- cut(as.numeric(sub("%", "", data.cb$tumor))/100, breaks = c(0,1/2, 1))

# Do not do internal standardization -- features have been standardized in the load_data.R script
standardize <- FALSE

# Reset data frame containing the validation results
results <- NULL

############################
###
### PRIM
###

print("PRIM")

lambda <- msgl.lambda.seq(x.prim, classes.prim, alpha = build$alpha, d = build$d, standardize = standardize, lambda.min = build$lambdamin)
fit <- msgl(x.prim, classes.prim, lambda = lambda, standardize = standardize, alpha = build$alpha)

res.cb <- predict(fit, x.cb)

err <- colSums(res.cb$classes != classes.cb)/length(classes.cb)
features <- sapply(fit$beta, function(beta) sum(colSums(abs(beta)) != 0))
loglike <- sapply(res.cb$response, function(r) -loglik_sum(r, classes.cb))

results <- rbind(results, data.frame(class = "all", err = err, features = features, loglike = loglike, n=0, classifier = "PRIM + CB"))

for(j in 1:length(cls.list)) {

sel <- classes.cb %in% cls.list[[j]]

err <- colSums(res.cb$classes[sel,] != classes.cb[sel])/sum(sel)
loglike <- sapply(res.cb$response, function(r) -loglik_sum(r[,sel], classes.cb[sel]))


results <- rbind(results, data.frame(class = names(cls.list)[j], err = err, features = features, loglike = loglike, n= 0, classifier = "PRIM + CB"))
}

## Tp errors
err.tp <- list()

sel <- classes.cb %in% cls.list$non.liver
errors.matrix <- res.cb$classes[sel, ] != classes.cb[sel]
err.tp <- sapply(levels(tp.cb), function(q) colMeans(errors.matrix[tp.cb[sel] == q, ]))

results <- rbind(results, data.frame(class = "low", err = err.tp[,1], features = features, loglike = loglike, n= 0, classifier = "PRIM + CB"))
results <- rbind(results, data.frame(class = "high", err = err.tp[,2], features = features, loglike = loglike, n= 0, classifier = "PRIM + CB"))



############################
###
### SIM non linear 
###

print("SIM NL")

sW.simnl <- sapply(classes.simnl, function(cls) sum(classes.prim == cls)/(length(classes.prim)+length(classes.cb))/sum(classes.simnl == cls))

lambda <- msgl.lambda.seq(x.simnl, classes.simnl, sampleWeights = sW.simnl, alpha = build$alpha, d = build$d, standardize = standardize, lambda.min = build$lambdamin)
fit <- msgl(x.simnl, classes.simnl, sampleWeights = sW.simnl, lambda = lambda, standardize = standardize, alpha = build$alpha)

res.cb <- predict(fit, x.cb)

err <- colSums(res.cb$classes != classes.cb)/length(classes.cb)
features <- sapply(fit$beta, function(beta) sum(colSums(abs(beta)) != 0))
loglike <- sapply(res.cb$response, function(r) -loglik_sum(r, classes.cb))

results <- rbind(results, data.frame(class = "all", err = err, features = features, loglike = loglike, n=0, classifier = "SIM NL + CB"))

for(j in 1:length(cls.list)) {

sel <- classes.cb %in% cls.list[[j]]

err <- colSums(res.cb$classes[sel,] != classes.cb[sel])/sum(sel)
loglike <- sapply(res.cb$response, function(r) -loglik_sum(r[,sel], classes.cb[sel]))


results <- rbind(results, data.frame(class = names(cls.list)[j], err = err, features = features, loglike = loglike, n= 0, classifier = "SIM NL + CB"))
}

## Tp errors
err.tp <- list()

sel <- classes.cb %in% cls.list$non.liver
errors.matrix <- res.cb$classes[sel, ] != classes.cb[sel]
err.tp <- sapply(levels(tp.cb), function(q) colMeans(errors.matrix[tp.cb[sel] == q, ]))

results <- rbind(results, data.frame(class = "low", err = err.tp[,1], features = features, loglike = loglike, n= 0, classifier = "SIM NL + CB"))
results <- rbind(results, data.frame(class = "high", err = err.tp[,2], features = features, loglike = loglike, n= 0, classifier = "SIM NL + CB"))


############################
###
### SIM linear 
###

print("SIM LI")

sW.simli <- sapply(classes.simli, function(cls) sum(classes.prim == cls)/(length(classes.prim)+length(classes.cb))/sum(classes.simli == cls))

lambda <- msgl.lambda.seq(x.simli, classes.simli, sampleWeights = sW.simli, alpha = build$alpha, d = build$d, standardize = standardize, lambda.min = build$lambdamin)
fit <- msgl(x.simli, classes.simli, sampleWeights = sW.simli, lambda = lambda, standardize = standardize, alpha = build$alpha)

res.cb <- predict(fit, x.cb)


err <- colSums(res.cb$classes != classes.cb)/length(classes.cb)
features <- sapply(fit$beta, function(beta) sum(colSums(abs(beta)) != 0))
loglike <- sapply(res.cb$response, function(r) -loglik_sum(r, classes.cb))

results <- rbind(results, data.frame(class = "all", err = err, features = features, loglike = loglike, n=0, classifier = "SIM LI + CB"))

for(j in 1:length(cls.list)) {

sel <- classes.cb %in% cls.list[[j]]

err <- colSums(res.cb$classes[sel,] != classes.cb[sel])/sum(sel)
loglike <- sapply(res.cb$response, function(r) -loglik_sum(r[,sel], classes.cb[sel]))


results <- rbind(results, data.frame(class = names(cls.list)[j], err = err, features = features, loglike = loglike, n = 0, classifier = "SIM LI + CB"))
}

## Tp errors
err.tp <- list()

sel <- classes.cb %in% cls.list$non.liver
errors.matrix <- res.cb$classes[sel, ] != classes.cb[sel]
err.tp <- sapply(levels(tp.cb), function(q) colMeans(errors.matrix[tp.cb[sel] == q, ]))

results <- rbind(results, data.frame(class = "low", err = err.tp[,1], features = features, loglike = loglike, n= 0, classifier = "SIM LI + CB"))
results <- rbind(results, data.frame(class = "high", err = err.tp[,2], features = features, loglike = loglike, n= 0, classifier = "SIM LI + CB"))

